// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import SwiftUI
import ArcGIS

private class Model: ObservableObject {

    // The analysis overlay to be added to the scene.
    let analysisOverlay = AnalysisOverlay()

    let viewshed: LocationViewshed

    var maxDistance: Double {
        didSet {
            viewshed.maxDistance = maxDistance
        }
    }

    init() {
        self.viewshed = LocationViewshed(
            location: Point(latitude: 0, longitude: 0),
            heading: 0,
            pitch: 90,
            horizontalAngle: 360,
            verticalAngle: 180,
            minDistance: 10,
            maxDistance: 12_000
        )
        viewshed.isVisible = false
        analysisOverlay.addAnalysis(viewshed)
        maxDistance = viewshed.maxDistance!
    }

    func setViewshedLocation(point: Point) {
        viewshed.location = point
        if !viewshed.isVisible {
            viewshed.isVisible = true
        }
    }

    func hideViewshed() {
        viewshed.isVisible = false
    }

}

struct ContentView: View {

    // An ObservableObject containing the scene, graphics overlay, and analysis overlay.
    @StateObject private var model = Model()

    // The Yosemite Valley hotspots scene.
    @State private var scene: ArcGIS.Scene = {

        let portalItem = PortalItem(
            portal: .arcGISOnline(connection: .anonymous),
            id: Item.ID("7558ee942b2547019f66885c44d4f0b1")!
        )

        return Scene(item: portalItem)
    }()


    var body: some View {

        SceneView(scene: scene, analysisOverlays: [model.analysisOverlay])

            .onLongPressGesture { _, scenePoint in
                guard let scenePoint = scenePoint else { return }
                model.setViewshedLocation(point: scenePoint)
            }

            .toolbar {
                ToolbarItemGroup(placement: .bottomBar) {

                    Slider(value: $model.maxDistance, in: 10...12_000)
                        .frame(width: 300)

                    Spacer()
                    Button("Clear") {
                        // Resets the line of sight.
                        model.hideViewshed()
                    }

                }
            }

    }

}


